#include "tmEventSetup/esAlgorithmHandle.hh"

#include "tmTable/tmTable.hh"

#include "tmUtil/tmUtil.hh"

#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>

namespace tmeventsetup
{

void
esAlgorithmHandle::replace(const std::string& from, const std::string& to)
{
  size_t p = 0;
  while ((p = expression_in_condition_.find(from, p)) != std::string::npos)
  {
    bool replace = true;
    const size_t index = p + from.length();
    if (index < expression_in_condition_.size())
    {
      switch (expression_in_condition_.at(index))
      {
        case ' ': break;
        case ')': break;
        default: replace = false;
      }
    }
    if (replace) expression_in_condition_.replace(p, from.length(), to);
    p += to.length();
    if (p >= expression_in_condition_.size()) break;
  }
}


void
esAlgorithmHandle::print() const
{
  print(*this);
}


void
esAlgorithmHandle::print(const esAlgorithm& algo)
{
  TM_LOG_INF("");
  std::cout << "  name: " << algo.getName() << "\n";
  std::cout << "  expression: " << algo.getExpression() << "\n";
  std::cout << "  expression_in_condition: " << algo.getExpressionInCondition() << "\n";
  std::cout << "  rpn_vector: " << boost::join(algo.getRpnVector(), ", ") << "\n";
  std::cout << "  index: " << algo.getIndex() << "\n";
  std::cout << "  module_id: " << algo.getModuleId() << "\n";
  std::cout << "  module_index: " << algo.getModuleIndex() << "\n";
#if defined(SWIG)
  std::cout << "  labels: " << boost::join(algo.getLabels(), ", ") << "\n";
#endif
}


void
esAlgorithmHandle::init(const tmtable::Row& algorithm)
{
  TM_LOG_DBG("algorithm.name=" <<TM_QUOTE(algorithm.at("name")));

  name_ = algorithm.at("name");
  expression_ = algorithm.at("expression");

  index_ = boost::lexical_cast<unsigned int>(algorithm.at("index"));
  module_id_ = boost::lexical_cast<unsigned int>(algorithm.at("module_id"));
  module_index_ = boost::lexical_cast<unsigned int>(algorithm.at("module_index"));

#if defined(SWIG)
  if (algorithm.count("labels"))
  {
    std::set<std::string> labels;
    std::vector<std::string> tokens;
    boost::split(tokens, algorithm.at("labels"), boost::is_any_of(","));
    std::for_each(tokens.begin(), tokens.end(), [](std::string& s){ boost::algorithm::trim(s); });
    for (const std::string& token: tokens)
    {
      labels.emplace(token);
    }
    setLabels(labels);
  }
#endif
}


void
esAlgorithmHandle::setExpressionInCondition(const std::map<std::string, std::string>& map)
{
  TM_LOG_DBG("");
  expression_in_condition_ = expression_;
  for (std::map<std::string, std::string>::const_iterator cit = map.begin();
       cit != map.end(); ++cit)
  {
    TM_LOG_DBG(TM_QUOTE(cit->first) << " : " << TM_QUOTE(cit->second));
    replace(cit->first, cit->second);
  }
}


#if defined(SWIG)
void
esAlgorithmHandle::setLabels(const std::set<std::string>& labels)
{
  labels_ = labels;
}
#endif

} // namespace tmeventsetup
